% NOTES:
%
% Just run and click the point to be tracked. The motion predictor will be automatically created.
%
% First 3 paragraphs just set learning options, so you need not to change it if you do not want to. Function makeModel2D(im, trData, xp)
% estimates an optimal sequential predictor of point xp, with desired properties specified in trData structure. Sequential predictor is a
% sequence of linear predictors. Optimal means the one which can predict the motion with the smallest computational complexity.
%
% Options should be set up for the particular type of application. For example you can change trData.method='minmax'. This will requires minutes
% to learn the predictor but computational complexity of the achieved sequential predictor is approximatelly 3 times smaller.
% Range of the predictor is automatically maximal considered range in trData.Ranges. Prediction precision (size of uncertainty region) is given by
% trData.eps (in pixels). Remember that learning assures convergence of the sequential predictor only on a training set and only inside the
% the predictors range. 
%
% Better predictions from the position out of the predictor's range are achieved by LSQ method, however, minimax method provides 3 times smaller
% computational complexity. Hence, you should have theoretically start 3 times closer to the target point due to 3 time higher frame-rate in
% real-time tracking applications. You want probably observe any speed-up in this "toy-example", because most of the time is spent with
% visualisation.




%% I/O stuff
addpath codes
trData.nameModel = 'leguan';                          % name of the object (just for identification).
trData.fName = 'leguan';                            % name of the output model
trData.imgModel = [trData.nameModel,'.jpg'];          % name of the image used for training

%% training set of monolitics predictors
trData.Z = [0.9:0.02:1.1];                            % zooms
trData.F = [-pi/20:pi/150:pi/20];                     % angles (radians)  
trData.noise = 0.1;                                   % adds a white noise 
trData.numOfExamples = 300;                          % Size of training set generated from the input image
trData.sc = [0.6];                                    % Use also under-sampled image
trData.blurDir = [0 45 90 135];                       % Use also blured image (direction of motion blur in degrees)
trData.blurSize = [3];                                % Size of motion blur.
trData.supportSelection = 'rnd';                      % 'rnd' ... random, 'lsq' ... iterated LSQ support set selection (time consuming).
trData.rndSamples = 1;                                % if (method = 'lsq')&(supportSelection = 'rnd'), then randomized sampling has rndSamples iterations
trData.err_rel = false;                               % if (method = 'lsq'), then err_rel=true increase weights of good predictions
trData.supportRangeRatio = 4;
% Total number of examples is length(trData.numOfExamples)*length(trData.sc)*length(trData.blurDir)*length(trData.blurSize), reasonable number is
% 5000-10000. 


%% sequence of predictors
trData.Ranges = [24 21 18 15 12 9 6 3];                  % Set of considered range. Maximal range is range of the learned predictor
trData.Complexities = [20:40:180];                    % Set of considered complexities. 
% Total number of learned monolitic predictors is length(trData.Ranges)*length(trData.Complexities). The finer is the grid the better are
% results, but the higher is the number of learned predictors. Hence, be carrefull in conjunction with time consuming learning (e.g. minmax).
                                                        
trData.eps = 2;                                       % size of desired uncertainty region, if not achieved it is automatically increased
                                                      % to an achieavable value.
trData.nextIterErr = 1;                               % 1 ...... all trainig data have to be covered, 
                                                      % 0.99 ... 1% of outliers in each iteration is automatically removed
trData.nextIterMargin = 0.1;                          % Margin between uncertainty region and the range of the following predictor                            
trData.method = 'minmax';                                % 'lsq' ... learning is much faster but do not provide the global minimum of the predictor complexity, 
                                                      % 'minmax' .... time consuming but leads to minimum complexity.
trData.iterBaseCompl = min(trData.Complexities)+0;    % next iteration initialisation penalty
trData.abs = false;                                   % just do not change
trData.visu = true;                                   % visualisation on/off


%% Selection of a reference point

im = double(rgb2gray(imread(trData.imgModel)))/255;
imshow(im);    
xp = ginput(1)';
close all;

%% Learning
tic 
model = makeModel2D(im, trData, xp);
dt = toc
model.trData = trData;
model.trData.time = dt;
save(trData.fName,'model');
